unit unit2;

{$mode objfpc}{$H+}

interface

uses
   Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
   MaskEdit, ComCtrls;

const
   nf_GRAPPAvar = 'GRAPPAvar.dat';
   nf_GrappaRRlyrae = 'GRAPPA_RRLyrae.dat';
   nf_GrappaCepheides = 'GRAPPA_Cepheides.dat';
   nf_Log             ='GRAPPAVar_Extracteur.log';
   max_GRAPPAVar = 9543807;
   smax_GRAPPAVar = '9543807';

   max_classe = 25;
   vari_class : Array[1..max_classe] of string = ('"ACV|CP|MCP|ROAM|ROAP|SXARI"','"ACYG"','"AGN"','"BCEP"',
                              '"BE|GCAS|SDOR|WR"','"CEP"','"CV"','"DSCT|GDOR|SXPHE"',
                              '"ECL"','"ELL"','"EP"','"LPV"','"MICROLENSING"','"RCB"','"RR"',
                              '"RS"','"S"','"SDB"','"SN"','"SOLAR_LIKE"','"SPB"','"SYST"','"WD"',
                              '"YSO"','"GALAXY"');
type

   { TForm1 }

   TForm1 = class(TForm)
      ButtonRecord: TButton;
      ButtonChercher: TButton;
      ButtonExportPrism: TButton;
      EditSourceID: TEdit;
      Label1: TLabel;
      Memo1: TMemo;
      StatusBar1: TStatusBar;
      procedure ButtonRecordClick(Sender: TObject);
      procedure ButtonChercherClick(Sender: TObject);
      procedure ButtonExportPrismClick(Sender: TObject);
      procedure FormCreate(Sender: TObject);
      procedure FormDestroy(Sender: TObject);
   private

   public

   end;
type

   TGRAPPAVar = bitpacked record
                           //         Size
                           // Unit   (byte) Definition
      Source_ID : QWord;   //          8    Unique within a particular Data Release. See https://dms.cosmos.esa.int/COSMOS/doc_fetch.php?id=2779219
      RA        : int32;   // mas      4    R.A.truncated to 1mas precision truncation: +10.123 = 10
      DEC       : int32;   // mas      4    Declination+90° truncated 1 mas precision truncation: +10.123 = 10
      in_vari_classifier_result,   // 1/8   Set if present in vari_clssifier_result table
      in_vari_rrlyrae,             // 1/8   Set if present in vari_rrlyrae table
      in_vari_cepheid,             // 1/8   Set if present in vari_cepheid table
      in_vari_planetary_transit,   // 1/8   Set if present in vari_planetary_transit table
      in_vari_short_time_scale,    // 1/8   Set if present in vari_short_time_scale table
      in_vari_long_period_variable,// 1/8   Set if present in vari_long_period_variable table
      in_vari_eclipsing_binary,    // 1/8   Set if present in vari_eclipsing_binary table
      in_vari_rotation_modulation, // 1/8   Set if present in vari_rotation_modulation table
      in_vari_ms_ocillator,        // 1/8   Set if present in vari_ms_ocillator table
      in_vari_agn,                 // 1/8   Set if present in vari_agn table
      in_vari_microlens,           // 1/8   Set if present in vari_microlens table
      in_vari_compact_companion,   // 1/8   Set if present in vari_compact_companion table
      dummy1,dummy2,dummy3,dummy4   : boolean;

      num_selected_g_fov,
      min_mag_g_fov,max_mag_g_fov,mean_mag_g_fov,median_mag_g_fov,    // magnitude*1000
      num_selected_bp,
      min_mag_bp,max_mag_bp,mean_mag_bp,median_mag_bp,                // magnitude*1000
      num_selected_rp,
      min_mag_rp,max_mag_rp,mean_mag_rp,median_mag_rp  : smallint;    // magnitude*1000

      best_class_score : byte;
      best_class_name  : shortInt;

   end;
   TFGrappaVar = bitpacked array[1..max_GRAPPAVar] of TGRAPPAVar;
var
   Form1: TForm1;
   RRLyrae_ok,Cepheides_ok : boolean;
   fGRAPPAVar : file of TFGRAPPAVar;
   pGRAPPAVar : ^TFGRAPPAVar;
   fLog       : TextFile;
implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.ButtonExportPrismClick(Sender: TObject);
begin

end;

procedure TForm1.ButtonChercherClick(Sender: TObject);
var
   Source_ID         : QWord;
   code,k            : integer;
   index,deb,fin     : longint;
   trouve            : boolean;
   sIndex,sRA,sDEC,sScore,
   s1,s2,s3,s4,s5    : string;
begin
   trouve:=false;
   Val(EditSourceID.Text,Source_id,code);
   if code<>0 then
      Memo1.Append(EditSourceID.Text+' n''est pas un Source_ID correct.')
   else
   begin
      index:=1;
      if source_ID=pGRAPPAVar^[index].Source_ID then trouve:=true
      else
      begin
         index:=max_GRAPPAVar;
         if source_ID=pGRAPPAVar^[index].Source_ID then trouve:=true
         else
         // On effectue une recherche par dicotomie :
         begin
            deb:=1; fin:=max_GRAPPAVar;
            k:=0;
            repeat
               index:=(deb+fin) div 2;
               if source_ID=pGRAPPAVar^[index].Source_ID then trouve:=true
               else
               begin
                  if pGRAPPAVar^[index].Source_ID<Source_id then deb:=index
                  else fin:=index;
                  inc(k);
               end;
            until trouve or (k>25);;
         end;
      end;
      Str(Index,sIndex);
      if trouve then
      with pGRAPPAVar^[index] do
      begin
         Memo1.Clear;
         Str(RA/3600000:11:6,sRA);
         Str((DEC/3600000-90):11:7,sDEC);
         Str(best_class_score/255:5:3,SScore);
         Memo1.Append('Source_ID : '+EditSourceID.Text{+' trouvé à la position n°'+sIndex});
         Memo1.Append('');
         Memo1.Append('RA = '+sRA+'  DEC = '+sDEC);
         Memo1.Append('');

         Memo1.Append('Classification : '+Vari_Class[best_class_name]+' score : '+sScore);
         Str(num_selected_g_fov:3,s1);
         Str(min_mag_g_fov/1000:6:3,s2);
         Str(max_mag_g_fov/1000:6:3,s3);
         Str(mean_mag_g_fov/1000:6:3,s4);
         Str(median_mag_g_fov/1000:6:3,s5);
         Memo1.Append('');
         Memo1.Append('       nb_obs  min     max     moyen   median');
         Memo1.Append('');
         Memo1.Append('G mag     '+s1+'  '+s2+'  '+s3+'  '+s4+'  '+s5);
         Str(num_selected_bp:3,s1);
         Str(min_mag_bp/1000:6:3,s2);
         Str(max_mag_bp/1000:6:3,s3);
         Str(mean_mag_bp/1000:6:3,s4);
         Str(median_mag_bp/1000:6:3,s5);
//         Memo1.Append('');
         Memo1.Append('BP mag    '+s1+'  '+s2+'  '+s3+'  '+s4+'  '+s5);

         Str(num_selected_rp:3,s1);
         Str(min_mag_rp/1000:6:3,s2);
         Str(max_mag_rp/1000:6:3,s3);
         Str(mean_mag_rp/1000:6:3,s4);
         Str(median_mag_rp/1000:6:3,s5);
//         Memo1.Append('');
         Memo1.Append('RP mag    '+s1+'  '+s2+'  '+s3+'  '+s4+'  '+s5);
         Memo1.Append('');
         if in_vari_classifier_result     then Memo1.Append('in_vari_classifier_result');
         if in_vari_rrlyrae               then Memo1.Append('in_vari_rrlyrae');
         if in_vari_cepheid               then Memo1.Append('in_vari_cepheid');
         if in_vari_planetary_transit     then Memo1.Append('in_vari_planetary_transit');
         if in_vari_short_time_scale      then Memo1.Append('in_vari_short_time_scale');
         if in_vari_long_period_variable  then Memo1.Append('in_vari_long_period_variable');
         if in_vari_eclipsing_binary      then Memo1.Append('in_vari_eclipsing_binary');
         if in_vari_rotation_modulation   then Memo1.Append('in_vari_rotation_modulation');
         if in_vari_ms_ocillator          then Memo1.Append('in_vari_ms_ocillator');
         if in_vari_agn                   then Memo1.Append('in_vari_agn');
         if in_vari_microlens             then Memo1.Append('in_vari_microlens');
         if in_vari_compact_companion     then Memo1.Append('in_vari_compact_companion');

{
// magnitude*1000
,
min_mag_bp,max_mag_bp,mean_mag_bp,median_mag_bp,                // magnitude*1000
num_selected_rp,
min_mag_rp,max_mag_rp,mean_mag_rp,median_mag_rp  : smallint;    // magnitude*1000


}
      end
      else
         Memo1.Append(EditSourceID.Text+' introuvable...')
   end;
end;

procedure TForm1.ButtonRecordClick(Sender: TObject);
var
   Rec_ID         : QWord;
   code,k            : integer;
   index,deb,fin     : longint;
   trouve            : boolean;
   sIndex,sRA,sDEC,sScore,
   s1,s2,s3,s4,s5    : string;
begin
   trouve:=false;
   Val(EditSourceID.Text,Rec_id,code);
   if code<>0 then
      Memo1.Append(EditSourceID.Text+' n''est pas un Rec_ID correct (1 à '+smax_GRAPPAVar+' SVP).')
   else
   begin
      index:=1;
      trouve:=false;
      if (REC_ID>0) and (REC_ID<=max_GRAPPAVar) then trouve:=true;
      Index:=REC_ID;
      Str(Index,sIndex);
      if trouve then
      with pGRAPPAVar^[index] do
      begin
         Memo1.Clear;
         Str(RA/3600000:11:6,sRA);
         Str((DEC/3600000-90):11:7,sDEC);
         Str(best_class_score/255:5:3,SScore);
         Str(source_id,s1);
         Memo1.Append('Source_ID : '+s1);
         Memo1.Append('');
         Memo1.Append('RA = '+sRA+'  DEC = '+sDEC);
         Memo1.Append('');

         Memo1.Append('Classification : '+Vari_Class[best_class_name]+' score : '+sScore);
         Str(num_selected_g_fov:3,s1);
         Str(min_mag_g_fov/1000:6:3,s2);
         Str(max_mag_g_fov/1000:6:3,s3);
         Str(mean_mag_g_fov/1000:6:3,s4);
         Str(median_mag_g_fov/1000:6:3,s5);
         Memo1.Append('');
         Memo1.Append('       nb_obs  min     max     moyen   median');
         Memo1.Append('');
         Memo1.Append('G mag     '+s1+'  '+s2+'  '+s3+'  '+s4+'  '+s5);
         Str(num_selected_bp:3,s1);
         Str(min_mag_bp/1000:6:3,s2);
         Str(max_mag_bp/1000:6:3,s3);
         Str(mean_mag_bp/1000:6:3,s4);
         Str(median_mag_bp/1000:6:3,s5);
//         Memo1.Append('');
         Memo1.Append('BP mag    '+s1+'  '+s2+'  '+s3+'  '+s4+'  '+s5);

         Str(num_selected_rp:3,s1);
         Str(min_mag_rp/1000:6:3,s2);
         Str(max_mag_rp/1000:6:3,s3);
         Str(mean_mag_rp/1000:6:3,s4);
         Str(median_mag_rp/1000:6:3,s5);
//         Memo1.Append('');
         Memo1.Append('RP mag    '+s1+'  '+s2+'  '+s3+'  '+s4+'  '+s5);
         Memo1.Append('');
         if in_vari_classifier_result     then Memo1.Append('in_vari_classifier_result');
         if in_vari_rrlyrae               then Memo1.Append('in_vari_rrlyrae');
         if in_vari_cepheid               then Memo1.Append('in_vari_cepheid');
         if in_vari_planetary_transit     then Memo1.Append('in_vari_planetary_transit');
         if in_vari_short_time_scale      then Memo1.Append('in_vari_short_time_scale');
         if in_vari_long_period_variable  then Memo1.Append('in_vari_long_period_variable');
         if in_vari_eclipsing_binary      then Memo1.Append('in_vari_eclipsing_binary');
         if in_vari_rotation_modulation   then Memo1.Append('in_vari_rotation_modulation');
         if in_vari_ms_ocillator          then Memo1.Append('in_vari_ms_ocillator');
         if in_vari_agn                   then Memo1.Append('in_vari_agn');
         if in_vari_microlens             then Memo1.Append('in_vari_microlens');
         if in_vari_compact_companion     then Memo1.Append('in_vari_compact_companion');

{
// magnitude*1000
,
min_mag_bp,max_mag_bp,mean_mag_bp,median_mag_bp,                // magnitude*1000
num_selected_rp,
min_mag_rp,max_mag_rp,mean_mag_rp,median_mag_rp  : smallint;    // magnitude*1000


}
      end
      else
         Memo1.Append(EditSourceID.Text+' introuvable...')
   end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
   RRLyrae_ok:=false;
   Cepheides_ok:=false;
   Memo1.Lines.Clear;

   AssignFile(fLog,nf_log);
   Rewrite(fLog);
   Writeln(fLog,DateTimeToStr(now),' GRAPPAVar Extracteur.');
   Writeln(fLog,DateTimeToStr(now));


   if not FileExists(nf_GrappaVar) then
   begin
     EditSourceID.Enabled:=false;
     EditSourceID.Text:='Fichier GRAPPAVar.dat introuvable !';
     ButtonChercher.Enabled:=false;
     Memo1.Append('Ce programme ne fonctionne que s''il est mis dans le même répertoire que le fichier GRAPPAVar.dat');
     StatusBar1.Panels.Items[0].Text:='GRAPPAVar.dat : KO';
     StatusBar1.Panels.Items[0].Width:=110;
     Writeln(fLog,DateTimeToStr(now),' ',nf_GRAPPAVar,' pas trouvé. L''application ne peut pas fonctionner.');

   end
   else
   begin
      Writeln(fLog,DateTimeToStr(now),' ',nf_GRAPPAVar,' trouvé.');
      Memo1.Append('Mettre un source_id GAIA DR3 dans le champ ci dessus et cliquer sur Chercher...');
      Writeln(fLog,DateTimeToStr(now),' Lecture de ',nf_GRAPPAVar);
      with StatusBar1.Panels.Items[0] do
      begin
         Text:='GRAPPAVar.dat : OK';
         Width:=115;
      end;

      New(pGRAPPAVar);
      Assignfile(fGRAPPAVar,nf_GrappaVar);
      reset(fGRAPPAVar);
      read(fGrappaVar,pGRAPPAVar^);


      if not FileExists(nf_GrappaRRlyrae) then
      begin
        with StatusBar1.Panels.Items[1] do
        begin
           Text:=nf_GRAPPARRlyrae+' : KO';
           Width:=145;
        end;
        Memo1.Append('La recherche des informations physiques des RR Lyrae n''est pas possible.');
        Writeln(fLog,DateTimeToStr(now),' ',nf_GRAPPARRlyrae+' non trouvé.');

      end else
      with StatusBar1.Panels.Items[1] do
      begin
         Text:=nf_GRAPPARRlyrae+' : OK';
         Width:=145;
         RRLyrae_ok:=true;
         Writeln(fLog,DateTimeToStr(now),' ',nf_GRAPPARRlyrae+' trouvé.');

      end;

      if not FileExists(nf_GrappaCepheides) then
      begin
        with StatusBar1.Panels.Items[2] do
        begin
           Text:=nf_GRAPPACepheides+' : KO';
           Width:=145;
        end;
        Memo1.Append('La recherche des informations physiques des Cepheides n''est pas possible.');
        Writeln(fLog,DateTimeToStr(now),' ',nf_GRAPPACepheides+' non trouvé.');
      end else
      with StatusBar1.Panels.Items[2] do
      begin
         Text:=nf_GRAPPACepheides+' : OK';
         Width:=145;
         Cepheides_ok:=true;
         Writeln(fLog,DateTimeToStr(now),' ',nf_GRAPPACepheides+' trouvé.');
      end;

   end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
   if FileExists(nf_GrappaVar) then Dispose(pGRAPPAVar);
   CloseFile(fLog);
end;

end.

